/*-----------------------------------------------------------------------------
|   File:      CC1110_UART.c
+----------------------------------------------------------------------------*/

/*==== DECLARATION CONTROL ===================================================*/
/*==== INCLUDES ==============================================================*/

#include "hal_types.h"
#include "hal_board.h"
#include "hal_cc8051.h"
#include "CC2530_UART.h"
#include "uart_queue.h"

/*==== PUBLIC FUNCTIONS ======================================================*/

/*==== TYPES =================================================================*/
/*==== EXPORTS ===============================================================*/

/*==== MACROS=================================================================*/

//#define INIT_TP1()   do { TP1 = PIN_OFF; IO_DIR_PORT_PIN(1, 6, IO_OUT); P1SEL &= ~0x40;} while (0)

char g_string_hex[] = {"0123456789ABCDEF"};

// Baudrate = 115.2 kbps (U0BAUD.BAUD_M = 216, U0GCR.BAUD_E = 11)
#define UART_BAUD_M  216 // changed because of tunning frequency
#define UART_BAUD_E  11

// UART Serial Communction Device ...
UartDevE  g_UartSerialDev;
UartDevE  g_UartCommDev;

#define SIZE_OF_UART_RX_BUFFER 100

/*==== PUBLIC FUNCTIONS ======================================================*/

// This function initializes the UART RX session, by simply enabling the
// corresponding UART interrupt, and leave the sample reception to the
// UART ISR shown in Figure 19. Before this function is called the
// application must initialize the UART peripheral according to the
// code shown in Figure 3, Figure 11, Figure 12, and Figure 13.
// The code implements the following steps:

// 1. Initialize the UART RX buffer index.
// 2. Clear UART RX Interrupt Flag (TCON.URXxIF = 0)
// 3. Enable UART RX and Interrupt (IEN0.URXxIE = 1, UxCSR.RE = 1)
// 4. Enable global interrupt (IEN0.EA = 1)

void uartStartRxForIsr(unsigned char uartNum)
{
    if (uartNum == 0)
    {
        URX0IF = 0;
        U0CSR |= 0x40;
        IEN0 |= 0x04;
    }
    else
    {
        URX1IF = 0;
        U1CSR |= 0x40;
        IEN0 |= 0x08;
    }
    IEN0 |= 0x80;
}

// The code implements the following steps:
// 1. Initialize the UART TX buffer index.
// 2. Clear UART TX Interrupt Flag (IRCON2.UTXxIF = 0.
// 3. Enable UART TX Interrupt (IEN2.UTXxIE = 1)
// 4. Send very first UART byte
// 5. Enable global interrupt (IEN0.EA = 1).

void uartStartTxForIsr(unsigned char uartNum)
{
    if (uartNum == 0)
    {
        UTX0IF = 0;
        IEN2 |= 0x04;
    }
    else
    {
        UTX1IF = 0;
        IEN2 |= 0x08;
    }
    IEN0 |= 0x80;
}

_Pragma("vector=0x13") __near_func __interrupt void UART0_RX_ISR(void);
_Pragma("vector=0x13") __near_func __interrupt void UART0_RX_ISR(void)
{
    URX0IF = 0;
    UartQueue_Rx_EnQueue(U0DBUF);
}

extern uint8   Uart0_Tx_End;

_Pragma("vector=0x3B") __near_func __interrupt void UART0_TX_ISR(void);
_Pragma("vector=0x3B") __near_func __interrupt void UART0_TX_ISR(void)
{
    UTX0IF = 0;

    if( UartQueue_Tx_Is_Empty() == TRUE )
    {
        Uart0_Tx_End = 1;
        IEN2 &= ~0x08;
        return;
    }
//    U0DBUF = uartTxBuffer[uartTxIndex++];
    U0DBUF = UartQueue_Tx_DeQueue();
}

void Uart1_Init(void)
{
    // Set UART 1 alternative 2
    PERCFG |= PERCFG_U1CFG;

    // Configure relevant Port P1 pins for peripheral function:
    P1SEL |= BIT7 | BIT6 ;
    
    // Initialize bitrate = 115.2 kbps 
    U1BAUD = UART_BAUD_M;
    U1GCR = (U1GCR&~UART_GCR_BAUD_E) | UART_BAUD_E;

    // Initialize UART protocol (start/stop bit, data bits, parity, etc.):

    // USART mode = UART (U0CSR.MODE = 1)
    U1CSR |= UART_CSR_MODE;

    // Start bit level = low => Idle level = high  (U0UCR.START = 0)
    U1UCR &= ~UART_UCR_START;

    // Stop bit level = high (U0UCR.STOP = 1)
    U1UCR |= UART_UCR_STOP;

    // Number of stop bits = 1 (U0UCR.SPB = 0)
    U1UCR &= ~UART_UCR_SPB;

    // Parity = disabled (U0UCR.PARITY = 0)
    U1UCR &= ~UART_UCR_PARITY;

    // 9-bit data enable = 8 bits transfer (U0UCR.BIT9 = 0)
//    U1UCR |= UART_UCR_BIT9;

    // Level of bit 9 = 0 (U0UCR.D9 = 0), used when U0UCR.BIT9 = 1
    // Level of bit 9 = 1 (U0UCR.D9 = 1), used when U0UCR.BIT9 = 1
    // Parity = Even (U0UCR.D9 = 0), used when U0UCR.PARITY = 1
    // Parity = Odd (U0UCR.D9 = 1), used when U0UCR.PARITY = 1
    U1UCR &= ~UART_UCR_D9;

    // Flow control = disabled (U0UCR.FLOW = 0)
    U1UCR &= ~UART_UCR_FLOW;

    // Bit order = LSB first (U0GCR.ORDER = 0)
    U1GCR &= ~UART_GCR_ORDER;
    // U1GCR |= UART_GCR_ORDER;
}

void Uart0_Init(void)
{
    // Set UART 0 alternative 1
    PERCFG &= ~(PERCFG_U0CFG);    

    // Configure relevant Port P0 pins for peripheral function:
    P0SEL |= BIT3 | BIT2;

    // Baudrate = 2.4 kbps (U0BAUD.BAUD_M = 59, U0GCR.BAUD_E = 6)
//    U0BAUD = 59;
//    U0GCR = (U0GCR&~UART_GCR_BAUD_E) | 6;
    // Baudrate = 9.6 kbps (U0BAUD.BAUD_M = 59, U0GCR.BAUD_E = 8)
//    U0BAUD = 59;
//    U0GCR = (U0GCR&~UART_GCR_BAUD_E) | 8;
    // Baudrate = 19.2 kbps (U0BAUD.BAUD_M = 59, U0GCR.BAUD_E = 9)
//    U0BAUD = 59;
//    U0GCR = (U0GCR&~UART_GCR_BAUD_E) | 9;
    // Baudrate = 28.8 kbps (U0BAUD.BAUD_M = 216, U0GCR.BAUD_E = 9)
//    U0BAUD = 216;
//    U0GCR = (U0GCR&~UART_GCR_BAUD_E) | 9;
    // Baudrate = 57.6 kbps (U0BAUD.BAUD_M = 216, U0GCR.BAUD_E = 10)
//    U0BAUD = 216;
//    U0GCR = (U0GCR&~UART_GCR_BAUD_E) | 10;    
    // Initialize bitrate = 115.2 kbps
    U0BAUD = 216;
    U0GCR = (U0GCR&~UART_GCR_BAUD_E) | 11;    

    // Initialize UART protocol (start/stop bit, data bits, parity, etc.):

    // USART mode = UART (U0CSR.MODE = 1)
    U0CSR |= UART_CSR_MODE;

    // Start bit level = low => Idle level = high  (U0UCR.START = 0)
    U0UCR &= ~UART_UCR_START;

    // Stop bit level = high (U0UCR.STOP = 1)
    U0UCR |= UART_UCR_STOP;

    // Number of stop bits = 1 (U0UCR.SPB = 0)
    U0UCR &= ~UART_UCR_SPB;

    // Parity = disabled (U0UCR.PARITY = 0)
    U0UCR &= ~UART_UCR_PARITY;

    // 9-bit data enable = 8 bits transfer (U0UCR.BIT9 = 0)
//    U0UCR |= UART_UCR_BIT9;

    // Level of bit 9 = 0 (U0UCR.D9 = 0), used when U0UCR.BIT9 = 1
    // Level of bit 9 = 1 (U0UCR.D9 = 1), used when U0UCR.BIT9 = 1
    // Parity = Even (U0UCR.D9 = 0), used when U0UCR.PARITY = 1
    // Parity = Odd (U0UCR.D9 = 1), used when U0UCR.PARITY = 1
    U0UCR &= ~UART_UCR_D9;

    // Flow control = disabled (U0UCR.FLOW = 0)
    U0UCR &= ~UART_UCR_FLOW;

    // Bit order = LSB first (U0GCR.ORDER = 0)
    U0GCR &= ~UART_GCR_ORDER;
    // U0GCR |= UART_GCR_ORDER;

    // UART interrupt setting ...
    UartQueue_Initialize();
    uartStartRxForIsr(0);
    uartStartTxForIsr(0);
}

uint8 Uart_Is_Active(UartDevE uartDev)
{
    if(UART_DEV_UART0 == uartDev)
    {
        return (U0CSR&UART_CSR_ACTIVE);
    }
    else /* UART_DEV_UART1 */
    {
        return (U1CSR&UART_CSR_ACTIVE);
    }
}

void Uart_Send_Char_NoWait(UartDevE uartDev, uint8 txChar)
{
    if(UART_DEV_UART0 == uartDev)
    {
//        U0CSR &= ~UART_CSR_TX_BYTE;
        U0DBUF = txChar;
    }
    else /* UART_DEV_UART1 */
    {
//        U1CSR &= ~UART_CSR_TX_BYTE;
        U1DBUF = txChar;
    }
}

void Uart_Send_Char(UartDevE uartDev, char txChar)
{
//    if( txChar == '\n') {
//        Uart_Send_Char(uartDev, '\r');
//    }

    if(UART_DEV_UART0 == uartDev)
    {
        U0CSR &= ~UART_CSR_TX_BYTE;
        U0DBUF = txChar;
        while(! (U0CSR&UART_CSR_TX_BYTE) );
    }
    else /* UART_DEV_UART1 */
    {
        U1CSR &= ~UART_CSR_TX_BYTE;
        U1DBUF = txChar;
        while(! (U1CSR&UART_CSR_TX_BYTE) );
    }
}

char Uart_Receive_Char(UartDevE uartDev)
{
    char rxChar;

    if(UART_DEV_UART0 == uartDev)
    {
        // Enable UART0 RX (U0CSR.RE = 1)
        U0CSR |= UART_CSR_RE;

        // Clear any pending RX interrupt request (set U0CSR.RX_BYTE = 0)
        U0CSR &= ~UART_CSR_RX_BYTE;

        // Wait until data received (U0CSR.RX_BYTE = 1)
        while( !(U0CSR&UART_CSR_RX_BYTE) );

        rxChar = (char)U0DBUF;
    }
    else /* UART_DEV_UART1 */
    {
        // Enable UART1 RX (U1CSR.RE = 1)
        U1CSR |= UART_CSR_RE;

        // Clear any pending RX interrupt request (set U0CSR.RX_BYTE = 0)
        U1CSR &= ~UART_CSR_RX_BYTE;

        // Wait until data received (U0CSR.RX_BYTE = 1)
        while( !(U1CSR&UART_CSR_RX_BYTE) );

        rxChar = (char)U1DBUF;
    }

    Uart_Send_Char(uartDev, rxChar);

    // Read UART0 RX buffer
    return (rxChar);
}

void Uart_Send_String(UartDevE uartDev, char * txChar_p)
{
    while((* txChar_p) != 0)
        Uart_Send_Char(uartDev, (* txChar_p++));
}

void Uart_Send_HexVal (UartDevE uartDev, unsigned int hexVal)
{
    char charString[11];
    unsigned int i;

    charString[0] = '0';
    charString[1] = 'x';

    for(i = 0; i < 8; i ++)
    {
        charString[i+2] = g_string_hex[ (hexVal >> (28 - (i*4))) & 0xF ];
    }
    charString[10] = 0;

    Uart_Send_String(uartDev, charString);
}

void Uart_Send_StringAndHexVal(UartDevE uartDev, char * txChar_p, unsigned int hexVal)
{
    Uart_Send_String(uartDev, txChar_p);
    Uart_Send_HexVal(uartDev, hexVal);
    Uart_Send_Char(uartDev, '\n');
}

/*==== END OF FILE ==========================================================*/
